########################################################################
# latex.mk
#
# Make production of LaTeX / PDF documents trivial, not just easy.
#
####################################
# Usage.
#
# In order to use this, simply define a variable PRODUCTS to specify
# your primary documents, like:
#
#   PRODUCTS = mydoc1.pdf mydoc2.pdf
#
# An unqualified "make" is equivalent to, "make all," which will produce
# all defined $(PRODUCTS) unless you define (or override) the ALL
# variable to be (e.g.) some subset of $(PRODUCTS), or perhaps include
# "packed" targets (see definition of same, below). Any member of
# products that can be produced by LaTeXMK will utilize same. You may
# also specify other items that will not be produced with LaTeXMK, but
# you must define those targets or the rules to generate them yourself.
#
# You may specify other variables controlling behavior at this point
# (see Customizable Variables (I)), such as PDFMODE (which defaults to,
# "-pdf"), e.g.:
#
#   PDFMDOE = -xelatex # or -luatex
#
# Then, include this make fragment with something like:
#
####################################
# ifneq (,$(SSDDOC_INC))
#   ldir = $(SSDDOC_INC)
# else
#   ldir = ../cet-is/ssddoc/include # As appropriate.
# endif
#
# include $(ldir)/latex.mk
####################################
#
# If the ssddoc UPS product is set up and/or SSDDOC_INC is defined in
# the environment or elsewhere, then SSDDOC_INC is used to find us,
# otherwise ldir should be set appropriately. The above example works if
# you have the cet-is repository checked out in the parent directory of
# the one in which your GNUmakefile resides.
#
# Following the include of this fragment, you may wish to override other
# customizable variables such as clean_dirs or clean_files (see
# Customizable Variables (II)), e.g.:
#
#   override clean_dirs += more dirs to clean
#   override clean_files += more files to clean
#   override clobber_files += more end products to clean
#
# See the Customizable Variables sections for variables that may be
# overridden, and what their default values might be, if any. Note that
# some variables (e.g. PANDOC_BEAMER) are most usefully overridden on a
# per-target basis (see notes, below)
#
# Any targets or additional dependencies you require should be defined
# after this file is included in order to avoid overriding all as the
# default target.
####################################
# Defined targets.
#
#   <item>:
#     Make the single product <item> (should be at least be a member of
#     the ALL list). Additionally <item> should be either a member of
#     PRODUCTS or defined by you implicitly or explicitly.
#
#   <item>-notes.pdf:
#     For PDF files produced from Markdown source only (via pandoc and
#     LaTeXMK): produce a version of <item>.pdf containing only notes
#     pages (see Beamer documentation of the "notes" class option).
#
#   <item>-packed.pdf:
#     For PDF files only, produce a packed version of <item> with
#     fonts embedded.
#
#   <item>-with_notes.pdf:
#     For PDF files produced from Markdown source only (via pandoc and
#     LaTeXMK): produce a version of <item> with slide notes
#     interspersed (see Beamer documentation of the "notes" class
#     option).
#
#   all:
#     Make all top-level targets that should be made by default.
#     Controlled by ALL (defaults to $(PRODUCTS)).
#
#   clean:
#     Remove all intermediate generated products, including those
#     handled by latexmk. Override clean_files and clean_dirs to add
#     files and directories to remove, respectively. Override ALL to
#     modify which target's intermediate products will be cleaned.
#
#   clobber:
#     Additionally remove final products. Override clobber_files to add
#     other files to remove. Override ALL to modify which final products
#     will be cleaned.
#
#   echo_<var>:
#     For debugging: print out the value of <var>, and its origin.
#
#   final-<item>:
#   final:
#     Obsolete targets (ambiguous name). Use packed-<item> and packed
#     targets, respectively.
#    
#   help:
#     Print all known targets.
#
#   macenv:
#     Add the LaTeX environment variables (TEXINPUTS, etc.) to the
#     Mac OS X GUI environment to enable applications such as
#     aquamacs started from the GUI to have the correct settings
#     for (e.g.) internal LaTeX invocation. Override tex_env_vars as
#     required (see make echo_tex_env_vars for current default).
#
#   packed:
#     Generate packed versions of PDF targets from ALL with fonts
#     embedded (see <item>-packed.pdf below for more details).
#
#   tidy:
#     Remove unwanted auxiliary files such as backups and patch
#     application products (.orig, .rej, etc.). Override tidy_files and
#     tidy_dirs as required (see make echo_tidy_<files|dirs> for current
#     defaults).
#
#   view:
#     Start the appropriate viewer for each item DVI, PDF or PS
#     item in $(ALL). Viewers are defined in ~/.latexmkrc---see LaTeXMK
#     documentation for details but can be overridden by specifying
#     PDFVIEWER, DVIVIEWER or PSVIEWER as appropriate. For example, to
#     set the pdf view to skim, ~/.latexmkrc should contain a line:
#
#       $pdf_previewer = "open -a skim %S"
#
#   view-<item>:
#     Run the defined viewer for <item> after first making sure that it
#     is up-to-date.
#
####################################
# Utility Functions.
#
#   $(call all_with <substring>,<list>)
#     Return a list of all members of <list> containing <substring>.
#
#   $(call all_without <substring>,<list>)
#     Return a list of all members of <list> *not* containing
#     <substring>.
#
#   $(call targets_with <substring>,<list>)
#     Return a unique list of all active top-level file targets of
#     KNOWN_FORMATS containing <substring>.
#
#   $(call targets_without <substring>,<list>)
#     Return a unique list of all active top-level file targets of
#     KNOWN_FORMATS *not* containing <substring>.
#
#   $(call uniq,<list>)
#     Returns a list of all unique elements of <list> in their original
#     order.
#
####################################
# Notes.
#
# * If you need to add targets (generation of some dependencies for
#   example, like figures), you should define that target after
#   including latex.mk, eg:
#
#     myFigure.tex: myFigure.dat
#
#   The latexmk machinery should take care of ensuring that it is
#   invoked at the correct time.
#
# * If you intend to have BibTeX autogenerate the .bbl file and would
#   like it cleaned up automatically, add "-bibtex" to
#   LATEXMK_EXTRA_OPTS.
#
# * If you are knitting a .pdf file from a .Rnw file, the .tex file will
#   be treated as an intermediate file and removed after the generation
#   of the .pdf file. In order to keep it around, either make it
#   explicitly (e.g. for debugging purposes) or add it to the .SECONDARY
#   special target in your local GNUmakefile.
#
# * Most targets are not echoed unless VERBOSE is set.
#
# * All output from tidy, clean and clobber targets is suppressed unless
#   DEBUG is set.
#
# * Standard output from LaTexMK runs will be suppressed if QUIET is
#   set. Standard error will be suppressed if QUIET is a number > 1.
#
# * When KnitR is invoked to generate an output file from a .Rnw file,
#   generated figures are placed in a target-specific
#   "genfigs-<target-file-stem>/" directory which is automatically added to
#   the clean_dirs variable.
#
# * When pandoc is invoked, the template used to produce a standalone
#   LaTeX document is specified in PANDOC_TEMPLATE. See make
#   echo_PANDOC_TEMPLATE for current default. See Customizable
#   Variables below for details of other ways of customizing the way
#   pandoc is used.
#
# * When customizing variables, note that they may be made target
#   specific with a line (e.g.):
#
#     <target>: override <VARIABLE> = value
#
# * If you need to specify multiple targets for a variable override, see
#   Utility Functions, above, for some useful functions to help. For
#   instance:
#
#     $(call targets_with,-talk): override PANDOC_BEAMER=YES
#
#   which sets PANDOC_BEAMER=YES for all targets containing
#   "-talk", including -with_notes and -notes derivative target; OR
#
#     $(call targets_without,-paper): override PANDOC_BEAMER=YES
#
#   which sets PANDOC_BEAMER=YES for all targets not containing
#   "-paper", including -with_notes and -notes derivative targets.
#
########################################################################
# Read-only variables (examine with "make echo_<var>").
#
# * LATEX_MK_DIR is set to the directory from which this file was
#   included, regardless of how it was found. Use it to set (e.g.)
#   TEXINPUTS, if appropriate.
#
# * LATEX_MK_VERSION is set to the "standard"-style version of this
#   makefile fragment (dotted).
#
# * LATEX_MK_UPS_VERSION is set to the UPS-style version of this makefile
#   fragment.
#
# * LATEX_MK_VERSION_NUM is set to the current numeric version of this
#   makefile fragment as a (large) number Mmmuu.
#
# * LATEX_MK_VERSION_PATCH is set to the current patchlevel (e.g. "a")
#   of this makefile fragment.
########################################################################
LATEX_MK_VERSION := 2.01.02

########################################################################
# Customizable Variables (I).
#
# These variables may be specified on the command line or in the
# environment, or overridden following the include of this make
# fragment.
########################################################################

# Contents of ALL target.
ifeq (,$(ALL))
  ALL = $(PRODUCTS)
endif

# Known top-level target file extensions.
ifeq (,$(KNOWN_FORMATS))
	KNOWN_FORMATS= %.docx %.dvi %.html %.md %.odt %.pdf %.pptx %.ps %.rtf
endif

# Default way to generate PDF from LaTeX source.
ifeq (,$(PDFMODE))
  PDFMODE = -pdf
endif

# latexmk executable.
ifeq (,$(LATEXMK))
	LATEXMK = latexmk
endif

# Extra latexmk command-line options.
ifeq (,$(LATEXMK_EXTRA_OPTS))
  LATEXMK_EXTRA_OPTS = -interaction=nonstopmode
endif

# Extra perl directives for latexmk.
ifeq (,$(LATEXMK_EXTRA_CMDS))
  LATEXMK_EXTRA_CMDS =
endif

# Viewers.
ifeq (,$(PDFVIEWER))
  PDFVIEWER = $(shell $(LATEXMK) -commands 2>&1 | \
   sed -ne 's/^.*view a pdf file,[^"]*"\([^"]*\)"$$/\1/p' | sed -e 's&%O&&')
endif
ifeq (,$(DVIVIEWER))
  DVIVIEWER = $(shell $(LATEXMK) -commands 2>&1 | \
   sed -ne 's/^.*view a dvi file,[^"]*"\([^"]*\)"$$/\1/p' | sed -e 's&%O&&')
endif
ifeq (,$(PSVIEWER))
  PSVIEWER = $(shell $(LATEXMK) -commands 2>&1 | \
   sed -ne 's/^.*view a ps file,[^"]*"\([^"]*\)"$$/\1/p' | sed -e 's&%O&&')
endif

# GhostScript executable.
ifeq (,$(GS))
  GS = gs
endif

# Rscript executable.
ifeq (,$(RSCRIPT))
  RSCRIPT = Rscript
endif

# Pandoc executable.
ifeq (,$(PANDOC))
  PANDOC=pandoc
endif

# Beamer for Pandoc. Most useful on a per-target basis.
ifeq (,$(PANDOC_BEAMER))
  PANDOC_BEAMER=
endif

# Pandoc Markdown options.
ifeq (,$(PANDOC_MARKDOWN_OPTS))
  PANDOC_MARKDOWN_OPTS = +autolink_bare_uris+ascii_identifiers+tex_math_single_backslash
endif

# Pandoc extra options.
ifeq (,$(PANDOC_EXTRA_OPTS))
  PANDOC_EXTRA_OPTS = $(if $(PANDOC_BEAMER),--slide-level 2 --highlight-style tango,)
endif

# Pandoc template.
ifeq (,$(PANDOC_TEMPLATE))
  PANDOC_TEMPLATE = $(if $(PANDOC_BEAMER),$(LATEX_MK_DIR)beamer/mfp.beamer,)
endif

# RMarkdown options.
ifeq (,$(RMARKDOWN_EXTRA_OPTS))
  RMARKDOWN_EXTRA_OPTS =
endif

# RMarkdown HTML output format.
ifeq (,$(RHTML_HTML_OUTPUT_FORMAT))
  RHTML_HTML_OUTPUT_FORMAT = html_document # Could be e.g. slidy_presentation.
endif

# RMarkdown Markdown output format.
ifeq (,$(RMARKDOWN_MD_OUTPUT_FORMAT))
  RMARKDOWN_MD_OUTPUT_FORMAT = md_document # Could be e.g. github_document.
endif

# RMarkdown PDF output format.
ifeq (,$(RMARKDOWN_PDF_OUTPUT_FORMAT))
  RMARKDOWN_PDF_OUTPUT_FORMAT = pdf_document # Could be e.g. beamer_presentation.
endif

# Echo.
ifeq (,$(ECHO))
  ECHO = @/bin/echo "==>"
endif

########################################################################
# Customizable Variables (II).
#
# These variables may be overridden following the include of this make
# fragment.
########################################################################

# Files removed by, "make tidy."
tidy_files =  \
  *.bak \
  *.orig \
  *.rej \
  *.old \
  *~ \
	*~[0-9] \
  *.rel

# Directories to be removed by make_tidy.
tidy_dirs = auto

# Extra directories to make sure we empty.
clean_dirs = $(DEPS_DIR)

# Extra files to make sure we remove.
clean_files =

# Extra files we should remove as part of clobber rather than clean.
clobber_files =

# Variables to push to environment with "make macenv"
tex_env_vars = \
  TEXINPUTS \
  BSTINPUTS \
  BIBINPUTS \
  MFINPUTS \
  MPINPUTS

########################################################################
# No user-serviceable parts below.
########################################################################

####################################
# Define "all" target early.

all: $(ALL)

####################################
# Variable definitions.

DEPS_DIR = .deps

ensure_deps = mkdir -p $(DEPS_DIR)

safe_fail = { $(RM) "$@"; false; }

def_maybe_pushenv = \
function maybe_pushenv() \
{ \
  local var; \
  for var in "$$@"; do \
    if [[ -n "$$var" ]] && [[ -n "$$(eval echo \"\$$$$var\")" ]] ; then \
      launchctl setenv "$$var" "$$(eval echo \"\$$$$var\")" && \
        echo "Pushed $$var to application env."; \
    else \
      true; \
    fi; \
  done; \
}

LATEX_MK_DIR := $(dir $(realpath $(lastword $(MAKEFILE_LIST))))
LATEX_MK_UPS_VERSION := $(shell echo "$(LATEX_MK_VERSION)" | perl -wne 'm&(\d+)(?:\.(\d+)(?:\.(\d+))?)?(?:[-.]?.*)$$& and printf "v%d_%02d_%02d%s",$${1},$${2} || 0, $${3} || 0, $${4} || "";')
LATEX_MK_VERSION_NUM := $(shell echo "$(LATEX_MK_VERSION)" | perl -wne 'm&(\d+)(?:\.(\d+)(?:\.(\d+))?)?& and printf "%d%02d%02d",$${1},$${2} || 0,$${3} || 0;')
LATEX_MK_VERSION_PATCH := $(shell echo "$(LATEX_MK_VERSION)" | perl -wpe 's&\d+(?:\.\d+(?:\.\d+)?)?[-.]?&&')

# Screen output control.
AT = $(if $(VERBOSE),,@)
SILENT_REDIRECT = $(if $(DEBUG),,>/dev/null 2>&1)
QUIET_REDIRECT = $(if $(QUIET),>/dev/null $(shell (( $(QUIET) > 1 )) && echo "2>&1"),)

# Utility.
uniq = $(if $1,$(firstword $1) $(call uniq,$(filter-out $(firstword $1),$1)))
all_with = $(foreach d,$(2),$(if $(findstring $(1),$(d)),$(d),))
all_without = $(foreach d,$(2),$(if $(findstring $(1),$(d)),,$(d)))
targets_with = $(call all_with,$(1),$(call uniq,$(filter-out view-%,$(filter $(KNOWN_FORMATS),$(ALL) $(MAKECMDGOALS)))))
targets_without = $(call all_without,$(1),$(call uniq,$(filter-out view-%,$(filter $(KNOWN_FORMATS),$(ALL) $(MAKECMDGOALS)))))

####################################
# Dependencies.
ifneq ($(filter clean clobber,$(MAKECMDGOALS)),)
  $(foreach file,$(PRODUCTS),$(eval -include $(DEPS_DIR)/$(file).d))
endif

####################################
# Targets
.PHONY: clean clobber macenv tidy cleanit cleandirs cleanfiles help

####################################
# LaTeXMK
LATEXMK_CMD = $(LATEXMK) -f- \
            -recorder -use-make -deps \
            $(LATEXMK_EXTRA_OPTS) \
            -e 'push @generated_exts, qw(nav snm vrb);' \
            -e '$(LATEXMK_EXTRA_CMDS)'

$(filter %.dvi, $(PRODUCTS)): %.dvi : %.tex
	@$(ensure_deps)
	$(ECHO) "Invoking $(LATEXMK) to generate $(@) from $(<)."
	$(AT)$(LATEXMK_CMD) -dvi -deps-out=$(DEPS_DIR)/$@.d $(<) $(QUIET_REDIRECT) || $(safe_fail)

$(filter-out $(foreach d,$(wildcard *.Rmd *.Rnw),$(d:.Rmd=.pdf)),$(filter %.pdf, $(PRODUCTS))): %.pdf : %.tex
	@$(ensure_deps)
	$(ECHO) "Invoking $(LATEXMK) to generate $(@) from $(<)."
	$(AT)$(LATEXMK_CMD) $(PDFMODE) -deps-out=$(DEPS_DIR)/$@.d $(<) $(QUIET_REDIRECT) || $(safe_fail)

$(filter %.ps, $(PRODUCTS)): %.ps : %.tex
	@$(ensure_deps)
	$(ECHO) "Invoking $(LATEXMK) to generate $(@) from $(<)."
	$(AT)$(LATEXMK_CMD) -ps -deps-out=$(DEPS_DIR)/$@.d $(<) $(QUIET_REDIRECT) || $(safe_fail)

cleanit: cleandirs cleanfiles

cleandirs:
	$(AT)$(RM) -r $(clean_dirs) $(SILENT_REDIRECT)

cleanfiles:
	$(if $(ALL),-$(AT)$(LATEXMK_CMD) $(clean_arg) -use-make- -deps- -f $(ALL) $(SILENT_REDIRECT),)
	$(AT)$(RM) $(clean_files) $(SILENT_REDIRECT)

clean: clean_arg = -c
clean: cleanit

tidy:
	$(AT)$(RM) $(tidy_files) $(SILENT_REDIRECT)
	$(AT)$(RM) -r $(tidy_dirs) $(SILENT_REDIRECT)

clobber: clean_arg = -C
clobber: cleanit
	$(AT)$(RM) $(clobber_files) $(ALL) \
             $(filter %.pdf,$(foreach d,$(ALL),$(d:.pdf=-packed.pdf))) \
             $(SILENT_REDIRECT)

macenv:
	$(AT)$(def_maybe_pushenv) $(SILENT_REDIRECT); maybe_pushenv $(tex_env_vars)

####################################
# View targets
.PHONY: view

ifneq (NONE,$(PDFVIEWER))
view-%.pdf : %.pdf
	$(ECHO) "Invoking \"$(patsubst %S,$(<),$(PDFVIEWER))\"."
	$(AT)$(patsubst %S,$(<),$(PDFVIEWER))
else
view-%.pdf : %.pdf
	$(ECHO) "PDF viewer not defined!"
	@false
endif

ifneq (NONE,$(DVIVIEWER))
view-%.dvi : %.dvi
	$(ECHO) "Invoking \"$(patsubst %S,$(<),$(DVIVIEWER))\"."
	$(AT)$(patsubst %S,$(<),$(DVIVIEWER))
else
view-%.dvi : %dvi
	$(ECHO) "DVI viewer not defined!"
	@false
endif

ifneq (NONE,$(PSVIEWER))
view-%.ps : %.ps
	$(ECHO) "Invoking \"$(patsubst %S,$(<),$(PSVIEWER))\"."
	$(AT)$(patsubst %S,$(<),$(PSVIEWER))
else
view-%.ps : %.ps
	$(ECHO) "PS viewer not defined!"
	@false
endif

view: $(foreach v,$(filter-out view-%, $(filter %.ps %.dvi %.pdf,$(ALL) $(MAKECMDGOALS))),view-$(v))
####################################

####################################
# Embed fonts for packed version.
$(filter %.pdf,$(foreach d,$(call uniq,$(ALL) $(MAKECMDGOALS)),$(d:.pdf=-packed.pdf))) : %-packed.pdf : %.pdf
	$(ECHO) "Reducing size of PDF for distribution $(<) -> $(@)."
	$(AT)$(GS) -dNOPAUSE -dBATCH -dQUIET \
    -dPrinted=false \
    -sDEVICE=pdfwrite \
    -dPDFSETTINGS=/prepress -dEmbedAllFonts=true -dSubsetFonts=true \
    -sOutputFile=$(@) -f $(<)

.PHONY: packed final

packed: $(filter %-packed.pdf,$(foreach d,$(filter-out %-packed.pdf,$(call uniq,$(ALL) $(MAKECMDGOALS))),$(d:.pdf=-packed.pdf)))

# Deal with bad legacy name.
ifneq (,$(findstring -final.pdf,$(ALL) $(MAKECMDGOALS)))
$(filter %-final.pdf,$(foreach d,$(filter-out %-final.pdf,$(call uniq,$(ALL) $(MAKECMDGOALS))),$(d:.pdf=-final.pdf))): %-final.pdf : %-packed.pdf
	$(ECHO) "WARNING: \"final\" targets are obsolete: generating $(<) instead."

.PHONY: $(filter %-final.pdf,$(foreach d,$(filter-out %-final.pdf,$(call uniq,$(ALL) $(MAKECMDGOALS))),$(d:.pdf=-final.pdf)))
final:
	$(ECHO) "WARNING: \"final\" is obsolete: use \"packed\" instead." 1>&2
endif
####################################

####################################
# KnitR / RMarkdown.
comma := ,

RMARKDOWN_CMD = $(RSCRIPT) -e \
'library("rmarkdown"); rmarkdown::render("$(<)", output_format="$(1)", output_file="$(@)"$(if $(RMARKDOWN_EXTRA_OPTS),$(comma) $(RMARKDOWN_EXTRA_OPTS),))' $(QUIET_REDIRECT) || $(safe_fail)

$(filter $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.docx)),$(PRODUCTS)): %.docx : %.Rmd
	$(ECHO) "Invoking $(RSCRIPT) to generate $(@) from $(<)."
	$(AT)$(call RMARKDOWN_CMD,word_document)

$(filter $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.html)),$(PRODUCTS)): %.html : %.Rmd
	$(ECHO) "Invoking $(RSCRIPT) to generate $(@) from $(<)."
	$(AT)$(call RMARKDOWN_CMD,$(RMARKDOWN_HTML_OUTPUT_FORMAT))

$(filter $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.md)),$(PRODUCTS)): %.md : %.Rmd
	$(ECHO) "Invoking $(RSCRIPT) to generate $(@) from $(<)."
	$(AT)$(call RMARKDOWN_CMD,$(RMARKDOWN_MD_OUTPUT_FORMAT))

$(filter $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.odt)),$(PRODUCTS)): %.odt : %.Rmd
	$(ECHO) "Invoking $(RSCRIPT) to generate $(@) from $(<)."
	$(AT)$(call RMARKDOWN_CMD,odt_document)

$(filter $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.pdf)),$(PRODUCTS)): %.pdf : %.Rmd
	$(ECHO) "Invoking $(RSCRIPT) to generate $(@) from $(<)."
	$(AT)$(call RMARKDOWN_CMD,$(RMARKDOWN_PDF_OUTPUT_FORMAT))

$(filter $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.pptx)),$(PRODUCTS)): %.pptx : %.Rmd
	$(ECHO) "Invoking $(RSCRIPT) to generate $(@) from $(<)."
	$(AT)$(call RMARKDOWN_CMD,powerpoint_presentation)

$(filter $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.rtf)),$(PRODUCTS)): %.rtf : %.Rmd
	$(ECHO) "Invoking $(RSCRIPT) to generate $(@) from $(<)."
	$(AT)$(call RMARKDOWN_CMD,rtf_document)

# Generated directories.
override clean_dirs += $(foreach stem, $(foreach d, $(wildcard *.Rmd), $(basename $(d))),$(stem)_cache $(stem)_files)

# In case someone decided to specify keep_tex: TRUE.
override clean_files += $(foreach d, $(wildcard *.Rmd),$(d:.Rmd=.tex))

%.tex : %.Rnw
	$(ECHO) "Invoking $(RSCRIPT) to generate $(@) from $(<)."
	$(AT)$(RSCRIPT) -e "library(\"knitr\"); \
knitr::opts_chunk\$$set(fig.path='genfigs-$(basename $(@))/', cache.path='$(basename $(@))_cache/'); \
knitr::knit('$(<)', output='$(@)')" $(QUIET_REDIRECT) || $(safe_fail)

# Intermediate targets, files and directories.
.INTERMEDIATE: $(foreach m,$(filter $(foreach d,$(wildcard *.Rnw),$(d:.Rnw=.pdf)),$(PRODUCTS)),$(m:.pdf=.tex))
override clean_files += $(foreach m,$(filter $(foreach d,$(wildcard *.Rnw),$(d:.Rnw=.pdf)),$(PRODUCTS)),$(m:.pdf=.tex))
override clean_dirs += $(foreach d,$(wildcard *.Rnw),genfigs-$(basename $(d)) $(basename $(d))_cache)
####################################

####################################
# Pandoc.
PANDOC_CMD = $(AT)$(PANDOC) -f markdown$(PANDOC_MARKDOWN_OPTS) \
$(if $(PANDOC_BEAMER),-t beamer,) \
$(if $(PANDOC_TEMPLATE),--template $(PANDOC_TEMPLATE),) \
--pdf-engine=pdflatex $(PANDOC_EXTRA_OPTS) \
$(<) -o $(@)

# Intermediate targets. Filter-out anything that might be related to
# RMarkdown since that will be dealt with via rmarkdown::render() above.
.INTERMEDIATE: $(foreach m,$(filter-out $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.md)),\
$(wildcard *.md)),$(m:.md=.tex) $(m:.md=-with_notes.tex) $(m:.md=-notes.tex))

# Add PANDOC_TEMPLATE as a dependency to all relevant .tex file targets.
$(foreach m,$(filter-out $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.md)),\
$(wildcard *.md)),$(m:.md=.tex) $(m:.md=-with_notes.tex) $(m:.md=-notes.tex)): $(PANDOC_TEMPLATE)

# Generate PDF from intermediate -with_notes.tex and -notes.tex files.
$(foreach m,$(filter-out $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.md)),\
$(wildcard *.md)),$(m:.md=-with_notes.pdf) $(m:.md=-notes.pdf)): %.pdf : %.tex
	@$(ensure_deps)
	$(ECHO) "Invoking $(LATEXMK) to generate $(@) from $(<)."
	$(AT)$(LATEXMK_CMD) $(PDFMODE) -deps-out=$(DEPS_DIR)/$@.d $(<) $(QUIET_REDIRECT) || $(safe_fail)

# Generate -with_notes.tex intermediate files from markdown source.
$(foreach m,$(filter-out $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.md)),\
$(wildcard *.md)),$(m:.md=-with_notes.tex)): %-with_notes.tex : %.md
	$(ECHO) "Invoking $(PANDOC) to generate $(@)$(if $(PANDOC_BEAMER), with beamer,)."
	$(PANDOC_CMD)

$(foreach m,$(filter-out $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.md)),\
$(wildcard *.md)), $(m:.md=-with_notes.tex)): \
override PANDOC_EXTRA_OPTS += -V classoption=notes

# Generate -notes.tex intermediate files from markdown source.
$(foreach m,$(filter-out $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.md)),\
$(wildcard *.md)), $(m:.md=-notes.tex)): %-notes.tex : %.md
	$(ECHO) "Invoking $(PANDOC) to generate $(@)$(if $(PANDOC_BEAMER), with beamer,)."
	$(PANDOC_CMD)

$(foreach m,$(filter-out $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.md)),\
$(wildcard *.md)), $(m:.md=-notes.tex)): \
override PANDOC_EXTRA_OPTS += -V classoption=notes=only

# Generate generic .tex intermediate files from markdown source.
%.tex : %.md
	$(ECHO) "Invoking $(PANDOC) to generate $(@)$(if $(PANDOC_BEAMER), with beamer,)."
	$(PANDOC_CMD)

# In case the user defines these intermediates as SECONDARY:
override clean_files += $(foreach m,$(filter-out $(foreach d,$(wildcard *.Rmd),$(d:.Rmd=.md)),\
$(wildcard *.md)),$(m:.md=.tex) $(m:.md=-notes.tex))
####################################

####################################
# Debugging
echo_%:
	$(ECHO) "$(subst echo_,,$@)=\"$($(subst echo_,,$@))\""
	$(ECHO) "origin $(subst echo_,,$@) returns $(origin $(subst echo_,,$@))"

help:
	$(AT)$(MAKE) -pRrq -f $(firstword $(MAKEFILE_LIST)) : 2>/dev/null | \
awk -v RS= -F: '/^# File/,/^# Finished Make data base/ {if ($$1 !~ "^[#.]") {print $$1}}' | \
sort | egrep -v -e '^[^[:alnum:]]' -e '^$@$$'
####################################

### Local Variables:
### mode: makefile-gmake
### End:
